@@ -4,7 +4,7 @@ PDL::Matrix -- a convenience matrix class for column-major access
 
 =head1 VERSION
 
-This document refers to version PDL::Matrix 0.01 of PDL::Matrix
+This document refers to version PDL::Matrix 0.5 of PDL::Matrix
 
 =head1 SYNOPSIS
 
@@ -33,13 +33,13 @@ want to read L<PDL::MatrixOps> or L<PDL::Slatec>.
 
 The original pdl class refers to the first index as the first row,
 the second index as the first column of a matrix. Consider
-     
+
   print $B = sequence(3,2)
   [
    [0 1 2]
    [3 4 5]
   ]
-  
+
 which gives a 2x3 matrix in terms of the matrix convention, but the
 constructor used (3,2). This might get more confusing when using
 slices like sequence(3,2)->slice("1:2,(0)") : with traditional
@@ -57,8 +57,8 @@ matrix. So now, the above example would be written as
    [4 5]
   ]
 
-Routines like L<eigens|eigens> or L<inv|inv> can be used without
-any changes.
+Routines like L<eigens|PDL::MatrixOps/eigens> or
+L<inv|PDL::MatrixOps/inv> can be used without any changes.
 
 Furthermore one can construct and use vectors as n x 1 matrices
 without mentioning the second index '1'.
@@ -68,8 +68,14 @@ without mentioning the second index '1'.
 C<PDL::Matrix> works by overloading a number of PDL constructors
 and methods such that first and second args (corresponding to
 first and second dims of corresponding matrices) are effectively swapped.
-It is not yet clear if PDL::Matrix achieves a consistent column
-major look-and-feel in this way.
+It is not yet clear if PDL::Matrix achieves a consistent column-major 
+look-and-feel in this way.
+
+=head1 NOTES
+
+As of version 0.5 (rewrite by CED) the matrices are stored in the usual
+way, just constructed and stringified differently.  That way indexing 
+and everything else works the way you think it should.
 
 =head1 FUNCTIONS
 
@@ -87,22 +93,34 @@ use Carp;
 
 @ISA = qw/PDL::Exporter PDL/;
 
-$VERSION = "0.02";
+$VERSION = "0.5";
 
 #######################################################################=
 #########
 #
 # overloads
 
+use overload( '""' => \&string,
+              'x'  => sub {my $foo = $_[0]->null();
+                           &PDL::Primitive::matmult(@_[1,0],$foo); 
+                           $foo;}
+            );
+
+sub string {
+    my ($me,@a) = shift;
+    return $me->SUPER::string(@a) unless($me->ndims > 0);
+    $me = $me->dummy(1,1) unless($me->ndims > 1);
+    $me->xchg(0,1)->SUPER::string(@a);
+}
+
+
 # --------> constructors
 
 =head2 mpdl, PDL::Matrix::pdl
 
 =for ref
 
-constructs an object of class PDL::Matrix which is a piddle child
-class, where the first index refers to the first column of the
-two-dimensional piddle.
+constructs an object of class PDL::Matrix which is a piddle child class.
 
 =for example
 
@@ -114,6 +132,10 @@ two-dimensional piddle.
 sub pdl {
   my $class = shift;
   my $pdl = $class->SUPER::pdl(@_);
+  if($pdl->ndims > 0) {
+      $pdl = $pdl->dummy(1,1) unless $pdl->ndims > 1;
+      $pdl = $pdl->xchg(0,1);
+  }
   bless $pdl, ref $class || $class;
 }
 
@@ -122,43 +144,13 @@ sub pdl {
 =for ref
 
 constructs a PDL::Matrix object similar to the piddle constructors
-zeroes, ones, sequence
+zeroes, ones, sequence.
 
 =cut
 
-# for constructors with specified dimensions I only have to swap these
-# dimensions
-for my $func (qw /zeroes ones sequence/) {
-  my $code = << "EOE";
-
-sub $func {
-  my \$class = shift;
-  my \@arg = \@_;
-  ref(\$arg[0]) ne 'PDL::Type' ? (\@arg >  1 ? (\@arg[1,0] = \@arg[0,1]) : 
-				  (\@arg[0,1] = (1,\$arg[0])) ) :
-	   (\@arg >  2 ? (\@arg[2,1] = \@arg[1,2]) :
-	    (\@arg[1,2] = (1,\$arg[1])) );
-  my \$pdl = \$class->SUPER::$func(\@arg);
-  bless \$pdl, ref \$class || \$class;
-}
-
-EOE
-  # print "evaluating $code\n";
-  eval $code;
-}
-
-# functions that construct a matrix pdl and that can be exported, very
-# trivial, they just call its methods optional type argument is
-# checked within these methods
-for my $func (qw /pdl zeroes ones sequence/) {
+for my $func (qw /pdl zeroes ones sequence dims/) {
   push @EXPORT_OK, "m$func";
-  my $code = << "EOE";
-
-sub m$func { PDL::Matrix->$func(\@_) }
-
-EOE
-# print "evaluating $code\n";
-  eval $code;
+  eval " sub m$func { PDL::Matrix->$func(\@_) }; ";
 }
 
 =head2 vpdl 
@@ -179,7 +171,7 @@ dimensions (n x 1)
 =cut 
 
 sub vpdl {
-  my $pdl = transpose(PDL->pdl(@_));
+  my $pdl = PDL->pdl(@_);
   bless $pdl, PDL::Matrix;
 }
 push @EXPORT_OK, "vpdl";
@@ -219,162 +211,8 @@ EOE
 
 
 
-=head2  PDL::Matrix::slice, PDL::Matrix::dice
-
-=for ref
-
-same as slice, dice for normal piddles, but reflecting the matrix
-convention by swapping the first two arguments.
-
-=for example
-
-    print  sequence(3,2)->slice("1:2,(0)") # piddle
-    [1 2]
-    print msequence(3,2)->slice("1:2,(0)") # PDL::Matrix
-    [2 4]
-
-=cut
-
-sub slice {
-  my $self = shift;
-  my $ind = shift;
-  # add another dimension if slices has only one
-  # convenient for vectors
-  $ind =~ s/^([^,]*)$/$1,/;
-  # swap first two arguments
-  $ind =~ s/^([^,]*),([^,]*)(.*)/$2,$1$3/;
-  $self->SUPER::slice($ind);
-}
-
-sub dice {
-  my $self = shift;
-  my @arg = @_;
-
-  
-  @arg >=2 ? @arg[1,0] = @arg[0,1] : ( (@arg == 1 && $self->dims == 2)  ?
-				       @arg = (0,$arg[0]) : 1 );
-  $self->SUPER::dice(@arg);
-}
-
-
-=head2 PDL::Matrix::at 
-
-=for ref
-
-same as at for piddles, but reflecting the matrix convention by
-swapping the first two arguments
-
-If only one scalar argument is used, we assume the object to be a
-vector and look only at the first column.
-
-=cut
-
-# swap arguments if number of arguments is greater than 1
-# if its one, look at the first column, assuming it is a vector
-sub at {
-  my $self = shift;
-  my @arg = @_;
-  @arg >=2 ? @arg[1,0] = @arg[0,1] : ( (@arg == 1 && $self->dims == 2)  ?
-       @arg = (0,$arg[0]) : 1 );
-  $self->SUPER::at(@arg);
-}
-
-=head2 PDL::Matrix::set
-
-=for ref
-
-set a particular value in a PDL::Matrix object. Note that this has to
-be called as an object method rather than a function
-
-=for example
-
-print msequence(3,3)->set(2,0,-1) # ok with PDL::Matrix convention
-[
- [ 0  1  2]
- [ 3  4  5]
- [-1  7  8]
-]
-
-print set msequence(3,3), 2,0,-1 # does not conform with PDL::Matrix convention
-[
- [ 0  1 -1]
- [ 3  4  5]
- [ 6  7  8]
-]
-
-
-=cut
-
-# this does not work as the usual set, but can be used via
-# $M->set(0,2,0.4) which sets the element in the first row and third
-# column to 0.4
-sub set{    # Sets a particular single value
-    barf("Usage: set($pdl, $x, $y,.., $value)") if $#_<2;
-    my $self  = shift; my $value = pop @_;
-    my @arg = @_;
-    @arg >=2 ? @arg[1,0] = @arg[0,1] : ( (@arg == 1 && $self->dims == 2)  ?
-					 @arg = (0,$arg[0]) : 1 );
-    PDL::Core::set_c ($self, [@arg], $value);
-    return $self;
-}
-
-=head2 PDL::Matrix::reshape
-
-=for ref
-
-same as reshape for piddles, but reflecting the matrix convention by
-swapping the first two arguments
-
-=cut
-
-sub reshape {
-  my $self = shift;
-  my @arg = @_;
-  @arg >=2 ? @arg[1,0] = @arg[0,1] : (@arg = (1,$arg[0]));
-  $self->SUPER::reshape(@arg);
-}
-
-# this is needed because only doing
-# > use overload '~' => \&PDL::transpose;
-# would not give an object of type PDL::Matrix back
-# => possible bug in transpose?!
-sub transpose {
-  my $self = shift;
-  my $pdl = $self->PDL::transpose;
-  bless $pdl, ref $self;
-}
-
-use overload '~' => \&PDL::Matrix::transpose;
-
-=head2 mdims
-
-=for ref
-
-returns the dimensions of the PDL::Matrix object in matrix
-convention
-
-C<dims> is NOT overloaded by PDL::Matrix to make sure that
-methods like PDL::transpose still work. So use C<mdims> to get
-the dims in the PDL::Matrix notation.
-
-=for example
-
-    print msequence(3,2)->mdims
-    3 2
-
-=cut
-
-# I cannot overload dims, because it is apparently used many times
-# in methods like transpose! :-(
-# this is not nice but I don't know how to avoid this
-sub mdims {
-  my $self = shift;
-  my @res = $self->SUPER::dims;
-  @res >=2 ? @res[1,0] = @res[0,1] : 1;
-  return @res;
-}
-
 eval "use PDL::Slatec";
+
 my $has_slatec = ($@ ? 0 : 1);
 sub inv {
   my $self = shift;
@@ -545,6 +383,13 @@ sub printTeX {
   print stringifyTeX(@_)."\n";
 }
 
+=pod 
+
+=begin comment
+
+DAL commented this out 17-June-2008. It didn't work, it used the
+outmoded (and incorrect) ~-is-transpose convention, and it wasn't
+necessary since the regular cross product worked fine.
 
 =head2  vcrossp, PDL::Matrix::crossp
 
@@ -552,16 +397,20 @@ sub printTeX {
 
 similar to PDL::crossp, however reflecting PDL::Matrix notations
 
-=cut
+#=cut
 
 # crossp for my special vectors
 sub crossp {
   my ($pdl1,$pdl2) = @_;
-  return transpose(PDL::crossp(~$pdl1,~$pdl2));
+  return PDL::transpose(PDL::crossp(~$pdl1,~$pdl2));
 }
 sub vcrossp { PDL::Matrix->crossp(\@_) }
 push @EXPORT_OK, "vcrossp";
 
+=end comment
+
+=cut
+
 %EXPORT_TAGS = (Func=>[@EXPORT_OK]);
 
 1;
@@ -573,6 +422,9 @@ operators may be applied to piddle-matrices. The inner product is not
 redefined. We might have missed some functions/methods. Internal
 consistency of our approach needs yet to be established.
 
+Because PDL::Matrix changes the way slicing behaves, it breaks many
+operators, notably those in MatrixOps.  
+
 =head1 TODO
 
 check all PDL functions, benchmarks, optimization, lots of other things ...
